@@ -1,4 +1,4 @@
-/* $Id: pjsua_app_legacy.c 4851 2014-05-23 09:29:09Z nanang $ */
+/* $Id: pjsua_app_legacy.c 5265 2016-03-17 03:37:13Z ming $ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -23,6 +23,20 @@
 
 #define THIS_FILE	"pjsua_app_legacy.c"
 
+
+/* An attempt to avoid stdout buffering for python tests:
+ * - call 'fflush(stdout)' after each call to 'printf()/puts()'
+ * - apply 'setbuf(stdout, 0)', but it is not guaranteed by the standard:
+ *   http://stackoverflow.com/questions/1716296
+ */
+#if (defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L) || \
+    (defined (_MSC_VER) && _MSC_VER >= 1400)
+/* Variadic macro is introduced in C99; MSVC supports it in since 2005. */
+#  define printf(...) {printf(__VA_ARGS__);fflush(stdout);}
+#  define puts(s) {puts(s);fflush(stdout);}
+#endif
+
+
 static pj_bool_t	cmd_echo;
 
 /*
@@ -38,9 +52,9 @@ static void print_buddy_list()
 
     pjsua_enum_buddies(ids, &count);
 
-    if (count == 0)
+    if (count == 0) {
 	puts(" -none-");
-    else {
+    } else {
 	for (i=0; i<(int)count; ++i) {
 	    pjsua_buddy_info info;
 
@@ -592,7 +606,7 @@ on_error:
 static void ui_make_new_call()
 {
     char buf[128];
-    pjsua_msg_data msg_data;
+    pjsua_msg_data msg_data_;
     input_result result;
     pj_str_t tmp;
 
@@ -617,10 +631,10 @@ static void ui_make_new_call()
 	tmp.slen = 0;
     }
 
-    pjsua_msg_data_init(&msg_data);
-    TEST_MULTIPART(&msg_data);
+    pjsua_msg_data_init(&msg_data_);
+    TEST_MULTIPART(&msg_data_);
     pjsua_call_make_call(current_acc, &tmp, &call_opt, NULL,
-			 &msg_data, &current_call);
+			 &msg_data_, &current_call);
 }
 
 static void ui_make_multi_call()
@@ -743,7 +757,7 @@ static void ui_answer_call()
 {
     pjsua_call_info call_info;
     char buf[128];
-    pjsua_msg_data msg_data;
+    pjsua_msg_data msg_data_;
 
     if (current_call != -1) {
 	pjsua_call_get_info(current_call, &call_info);
@@ -775,7 +789,7 @@ static void ui_answer_call()
 	if (st_code < 100)
 	    return;
 
-	pjsua_msg_data_init(&msg_data);
+	pjsua_msg_data_init(&msg_data_);
 
 	if (st_code/100 == 3) {
 	    if (!simple_input("Enter URL to be put in Contact",
@@ -784,7 +798,7 @@ static void ui_answer_call()
 	    hvalue = pj_str(contact);
 	    pjsip_generic_string_hdr_init2(&hcontact, &hname, &hvalue);
 
-	    pj_list_push_back(&msg_data.hdr_list, &hcontact);
+	    pj_list_push_back(&msg_data_.hdr_list, &hcontact);
 	}
 
 	/*
@@ -798,7 +812,7 @@ static void ui_answer_call()
 	    return;
 	}
 
-	pjsua_call_answer2(current_call, &call_opt, st_code, NULL, &msg_data);
+	pjsua_call_answer2(current_call, &call_opt, st_code, NULL, &msg_data_);
     }
 }
 
@@ -1065,7 +1079,7 @@ static void ui_call_transfer(pj_bool_t no_refersub)
 	pj_str_t STR_FALSE = { "false", 5 };
 	pjsua_call_info ci;
 	input_result result;
-	pjsua_msg_data msg_data;
+	pjsua_msg_data msg_data_;
 
 	pjsua_call_get_info(current_call, &ci);
 	printf("Transferring current call [%d] %.*s\n", current_call,
@@ -1080,26 +1094,26 @@ static void ui_call_transfer(pj_bool_t no_refersub)
 	    return;
 	}
 
-	pjsua_msg_data_init(&msg_data);
+	pjsua_msg_data_init(&msg_data_);
 	if (no_refersub) {
 	    /* Add Refer-Sub: false in outgoing REFER request */
 	    pjsip_generic_string_hdr_init2(&refer_sub, &STR_REFER_SUB,
 		&STR_FALSE);
-	    pj_list_push_back(&msg_data.hdr_list, &refer_sub);
+	    pj_list_push_back(&msg_data_.hdr_list, &refer_sub);
 	}
 	if (result.nb_result != PJSUA_APP_NO_NB) {
-	    if (result.nb_result == -1 || result.nb_result == 0)
+	    if (result.nb_result == -1 || result.nb_result == 0) {
 		puts("You can't do that with transfer call!");
-	    else {
+	    } else {
 		pjsua_buddy_info binfo;
 		pjsua_buddy_get_info(result.nb_result-1, &binfo);
-		pjsua_call_xfer( current_call, &binfo.uri, &msg_data);
+		pjsua_call_xfer( current_call, &binfo.uri, &msg_data_);
 	    }
 
 	} else if (result.uri_result) {
 	    pj_str_t tmp;
 	    tmp = pj_str(result.uri_result);
-	    pjsua_call_xfer( current_call, &tmp, &msg_data);
+	    pjsua_call_xfer( current_call, &tmp, &msg_data_);
 	}
     }
 }
@@ -1116,7 +1130,7 @@ static void ui_call_transfer_replaces(pj_bool_t no_refersub)
 	pj_str_t STR_FALSE = { "false", 5 };
 	pjsua_call_id ids[PJSUA_MAX_CALLS];
 	pjsua_call_info ci;
-	pjsua_msg_data msg_data;
+	pjsua_msg_data msg_data_;
 	char buf[128];
 	unsigned i, count;
 
@@ -1175,17 +1189,17 @@ static void ui_call_transfer_replaces(pj_bool_t no_refersub)
 	    return;
 	}
 
-	pjsua_msg_data_init(&msg_data);
+	pjsua_msg_data_init(&msg_data_);
 	if (no_refersub) {
 	    /* Add Refer-Sub: false in outgoing REFER request */
 	    pjsip_generic_string_hdr_init2(&refer_sub, &STR_REFER_SUB,
 					   &STR_FALSE);
-	    pj_list_push_back(&msg_data.hdr_list, &refer_sub);
+	    pj_list_push_back(&msg_data_.hdr_list, &refer_sub);
 	}
 
 	pjsua_call_xfer_replaces(call, dst_call,
 				 PJSUA_XFER_NO_REQUIRE_REPLACES,
-				 &msg_data);
+				 &msg_data_);
     }
 }
 
@@ -1201,8 +1215,13 @@ static void ui_send_dtmf_2833()
 	pj_status_t status;
 	char buf[128];
 
+#if defined(PJMEDIA_HAS_DTMF_FLASH) && PJMEDIA_HAS_DTMF_FLASH!= 0	    	
 	if (!simple_input("DTMF strings to send (0-9*R#A-B)", buf,
 	    sizeof(buf)))
+#else
+	if (!simple_input("DTMF strings to send (0-9*#A-B)", buf,
+	    sizeof(buf)))
+#endif
 	{
 	    return;
 	}
@@ -1248,19 +1267,19 @@ static void ui_send_dtmf_info()
 	digits = pj_str(buf);
 	for (i=0; i<digits.slen; ++i) {
 	    char body[80];
-	    pjsua_msg_data msg_data;
+	    pjsua_msg_data msg_data_;
 
-	    pjsua_msg_data_init(&msg_data);
-	    msg_data.content_type = pj_str("application/dtmf-relay");
+	    pjsua_msg_data_init(&msg_data_);
+	    msg_data_.content_type = pj_str("application/dtmf-relay");
 
 	    pj_ansi_snprintf(body, sizeof(body),
 		"Signal=%c\r\n"
 		"Duration=160",
 		buf[i]);
-	    msg_data.msg_body = pj_str(body);
+	    msg_data_.msg_body = pj_str(body);
 
 	    status = pjsua_call_send_request(current_call, &SIP_INFO,
-		&msg_data);
+					     &msg_data_);
 	    if (status != PJ_SUCCESS) {
 		return;
 	    }
@@ -1669,6 +1688,7 @@ static void ui_call_redirect(char menuin[])
     }
 }
 
+
 /*
  * Main "user interface" loop.
  */
