@@ -1,4 +1,4 @@
-/* $Id: sip_msg.c 4849 2014-05-23 03:24:55Z nanang $ */
+/* $Id: sip_msg.c 5191 2015-10-23 09:50:16Z nanang $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -1220,8 +1220,8 @@ static int pjsip_contact_hdr_print( pjsip_contact_hdr *hdr, char *buf,
 	char *endbuf = buf + size;
 
 	copy_advance(buf, (*hname));
-	*buf++ = ':';
-	*buf++ = ' ';
+	copy_advance_char_check(buf, ':');
+	copy_advance_char_check(buf, ' ');
 
 	printed = pjsip_uri_print(PJSIP_URI_IN_CONTACT_HDR, hdr->uri, 
 				  buf, endbuf-buf);
@@ -1702,17 +1702,21 @@ static int pjsip_routing_hdr_print( pjsip_routing_hdr *hdr,
     /* Check the proprietary param 'hide', don't print this header 
      * if it exists in the route URI.
      */
-    sip_uri = (pjsip_sip_uri*) pjsip_uri_get_uri(hdr->name_addr.uri);
-    p = sip_uri->other_param.next;
-    while (p != &sip_uri->other_param) {
-	const pj_str_t st_hide = {"hide", 4};
+    if (PJSIP_URI_SCHEME_IS_SIPS(hdr->name_addr.uri) ||
+	PJSIP_URI_SCHEME_IS_SIP(hdr->name_addr.uri))
+    {
+	sip_uri = (pjsip_sip_uri*) pjsip_uri_get_uri(hdr->name_addr.uri);
+	p = sip_uri->other_param.next;
+	while (p != &sip_uri->other_param) {
+	    const pj_str_t st_hide = {"hide", 4};
 
-	if (pj_stricmp(&p->name, &st_hide) == 0) {
-	    /* Check if param 'hide' is specified without 'lr'. */
-	    pj_assert(sip_uri->lr_param != 0);
-	    return 0;
+	    if (pj_stricmp(&p->name, &st_hide) == 0) {
+		/* Check if param 'hide' is specified without 'lr'. */
+		pj_assert(sip_uri->lr_param != 0);
+		return 0;
+	    }
+	    p = p->next;
 	}
-	p = p->next;
     }
 
     /* Route and Record-Route don't compact forms */
